:root {

    /*font sizes*/
    --pst-font-size-base: 17px;
    // heading font sizes
    --pst-font-size-h1: 32px;
    --pst-font-size-h2: 29px;
    --pst-font-size-h3: 26px;
    --pst-font-size-h4: 23px;
    --pst-font-size-h5: 21px;
    --pst-font-size-h6: 19px;

    /*    sphinx-design-color*/
    --sd-color-primary: rgb(0 167 222);
}

/*remove underline links*/
a {
    text-decoration: none;
}

/*******************************************************************************
* light theme
*
* all the variables used for light theme coloring
*/
html[data-theme="light"] {
    /*****************************************************************************
    * main colors
    */
    --pst-color-primary: rgb(0 167 222);
    --pst-color-secondary: rgb(238 144 64);
    --pst-color-success: rgb(40, 167, 69);
    --pst-color-info: var(--pst-color-primary);
    --pst-color-warning: var(--pst-color-secondary);
    --pst-color-danger: rgb(220, 53, 69);
    --pst-color-text-base: rgb(51, 51, 51);
    --pst-color-text-muted: rgb(77, 77, 77);
    --pst-color-border: rgb(201, 201, 201);
    --pst-color-shadow: rgb(216, 216, 216);

    /*****************************************************************************
    * depth colors
    *
    * background: the more in depth color
    * on-background: the object directly set on the background, use of shadows in light theme
    * surface: object set on the background (without shadows)
    * on_surface: object set on surface object (without shadows)
    */
    --pst-color-background: rgb(255, 255, 255);
    --pst-color-on-background: rgb(255, 255, 255);
    --pst-color-surface: rgb(240, 240, 240);
    --pst-color-on-surface: rgb(255, 255, 238);

    /*****************************************************************************
    * extentions
    */

    --pst-color-panel-background: var(--pst-color-background);

    /*****************************************************************************
    * layout
    */

    // links
    --pst-color-link: var(--pst-color-primary);
    --pst-color-link-hover: var(--pst-color-secondary);

    // inline code
    --pst-color-inline-code: rgb(232, 62, 140);

    // targeted content
    --pst-color-target: rgb(251, 229, 78);

    // hide any content that should not be displayed in the light theme
    .only-dark {
    display: none !important;
    }
}

/*******************************************************************************
* dark theme
*
* all the variables used for dark theme coloring
*/
html[data-theme="dark"] {
    /*****************************************************************************
    * main colors
    */
    --pst-color-primary: rgb(0 167 222);
    --pst-color-secondary: rgb(238 144 64);
    --pst-color-success: rgb(72, 135, 87);
    --pst-color-info: var(--pst-color-primary);
    --pst-color-warning: var(--pst-color-secondary);
    --pst-color-danger: rgb(203, 70, 83);
    --pst-color-text-base: rgb(201, 209, 217);
    --pst-color-text-muted: rgb(192, 192, 192);
    --pst-color-border: rgb(192, 192, 192);
    --pst-color-shadow: var(--pst-color-background);


    /*****************************************************************************
    * depth colors
    *
    * background: the more in depth color
    * on-background: the object directly set on the background, use of a light grey in dark theme
    * surface: object set on the background (without shadows)
    * on_surface: object set on surface object (without shadows)
    */
    --pst-color-background: rgb(18, 18, 18);
    --pst-color-on-background: rgb(30, 30, 30);
    --pst-color-surface: rgb(41, 41, 41);
    --pst-color-on-surface: rgb(55, 55, 55);

    /*****************************************************************************
    * extentions
    */

    --pst-color-panel-background: var(--pst-color-background-up);

    /*****************************************************************************
    * layout
    */

    // links
    --pst-color-link: var(--pst-color-primary);
    --pst-color-link-hover: var(--pst-color-secondary);

    // inline code
    --pst-color-inline-code: rgb(221, 158, 194);

    // targeted content
    --pst-color-target: rgb(71, 39, 0);

    // hide any content that should not be displayed in the dark theme
    .only-light {
    display: none !important;
    }

    // specific brightness applied on images
    img {
    filter: brightness(0.8) contrast(1.2);
    }
}
